@@ -43,6 +43,8 @@ typedef enum _HklBinocularsDetectorEnum
         HKL_BINOCULARS_DETECTOR_MERLIN_MEDIPIX_3RX_QUAD_512,
         /* Add new your detectors here */
         HKL_BINOCULARS_DETECTOR_NUM_DETECTORS,
+        /* wip */
+        HKL_BINOCULARS_DETECTOR_CIRPAD,
 } HklBinocularsDetectorEnum;
 
 HKLAPI extern int hkl_binoculars_detector_2d_number_of_detectors(void);
@@ -113,6 +115,8 @@ HKLAPI extern HklBinocularsSpace *hkl_binoculars_space_new(size_t n_indexes_0,
 
 HKLAPI extern void hkl_binoculars_space_free(HklBinocularsSpace *self);
 
+HKLAPI extern void hkl_binoculars_space_fprintf(FILE *f, const HklBinocularsSpace *space);
+
 /********/
 /* Cube */
 /********/
@@ -130,6 +134,8 @@ HKLAPI extern HklBinocularsCube *hkl_binoculars_cube_new_empty_from_cube(const H
 
 HKLAPI extern HklBinocularsCube *hkl_binoculars_cube_new_copy(const HklBinocularsCube *src);
 
+HKLAPI extern HklBinocularsCube *hkl_binoculars_cube_new_from_file(const char *fname);
+
 HKLAPI extern HklBinocularsCube *hkl_binoculars_cube_new_from_space(const HklBinocularsSpace *space);
 
 HKLAPI extern HklBinocularsCube *hkl_binoculars_cube_new_merge(const HklBinocularsCube *cube1,
@@ -138,12 +144,20 @@ HKLAPI extern HklBinocularsCube *hkl_binoculars_cube_new_merge(const HklBinocula
 HKLAPI extern unsigned int hkl_binoculars_cube_cmp(const HklBinocularsCube *self,
                                                    const HklBinocularsCube *other);
 
+HKLAPI extern void hkl_binoculars_cube_add_cube(HklBinocularsCube *self,
+                                                const HklBinocularsCube *cube);
+
 HKLAPI extern void hkl_binoculars_cube_add_space(HklBinocularsCube *self,
                                                  const HklBinocularsSpace *space);
 
-HKLAPI extern void hkl_binoculars_cube_save_hdf5(const char *fn,
-                                                 const char *config,
-                                                 const HklBinocularsCube *self);
+HKLAPI extern int hkl_binoculars_cube_save_hdf5(const char *fn,
+                                                const char *config,
+                                                const HklBinocularsCube *self);
+
+HKLAPI extern void hkl_binoculars_cube_merge_and_save_hdf5(const char *fn,
+                                                           const char *config,
+                                                           const char *fnames[],
+                                                           size_t n_fnames);
 
 HKLAPI extern void hkl_binoculars_cube_fprintf(FILE *f, const HklBinocularsCube *self);
 
@@ -152,7 +166,7 @@ HKLAPI extern void hkl_binoculars_cube_fprintf(FILE *f, const HklBinocularsCube
 /***************/
 
 /* subprojection for QCustom */
-/* q_index / tth_index  ( index = numero du scan) (optional) */
+/* q_scannumber / tth_scannumber  ( scannumber = numero du scan) (optional) */
 /* q_timescan0 / tth_timescan0   (timescan0 = timestamp du 1er point du scan (constant pour 1 scan)) */
 
 /* Adding the polarization correction from fit2D */
@@ -207,6 +221,8 @@ typedef enum _HklBinocularsQCustomSubProjectionEnum
         HKL_BINOCULARS_QCUSTOM_SUB_PROJECTION_QY_QZ_TIMESTAMP,
         HKL_BINOCULARS_QCUSTOM_SUB_PROJECTION_TTH_AZIMUTH,
         HKL_BINOCULARS_QCUSTOM_SUB_PROJECTION_Q_TIMESCAN0,
+        HKL_BINOCULARS_QCUSTOM_SUB_PROJECTION_Q_SCANNUMBER,
+        HKL_BINOCULARS_QCUSTOM_SUB_PROJECTION_TTH_SCANNUMBER,
         /* Add new your subprojection in the same order than the haskell order here */
         HKL_BINOCULARS_QCUSTOM_NUM_SUBPROJECTIONS,
 } HklBinocularsQCustomSubProjectionEnum;
@@ -228,6 +244,7 @@ typedef enum _HklBinocularsQCustomSubProjectionEnum
                                                        size_t n_limits, \
                                                        double timestamp, \
                                                        double timescan0, \
+                                                       int scannumber,  \
                                                        const HklBinocularsQCustomSubProjectionEnum subprojection, \
                                                        double uqx,      \
                                                        double uqy,      \
