@@ -1,11 +1,14 @@
 {-# LANGUAGE CPP                        #-}
 {-# LANGUAGE DataKinds                  #-}
+{-# LANGUAGE DeriveAnyClass             #-}
 {-# LANGUAGE DeriveGeneric              #-}
+{-# LANGUAGE DerivingStrategies         #-}
 {-# LANGUAGE FlexibleInstances          #-}
 {-# LANGUAGE GADTs                      #-}
 {-# LANGUAGE GeneralizedNewtypeDeriving #-}
 {-# LANGUAGE LambdaCase                 #-}
 {-# LANGUAGE OverloadedStrings          #-}
+{-# LANGUAGE PolyKinds                  #-}
 {-# LANGUAGE StandaloneDeriving         #-}
 {-# LANGUAGE TemplateHaskell            #-}
 {-# LANGUAGE TypeApplications           #-}
@@ -25,14 +28,11 @@
 
 module Hkl.Binoculars.Config
     ( Angstrom(..)
-    , Args
     , Attenuation(..)
     , BinocularsPreConfig(..)
     , Capabilities(..)
-    , Config
     , ConfigContent(..)
     , ConfigRange(..)
-    , DataPath
     , Degree(..)
     , DestinationTmpl(..)
     , FieldEmitter(..)
@@ -52,7 +52,6 @@ module Hkl.Binoculars.Config
     , Resolutions(..)
     , RLimits(..)
     , SampleAxis(..)
-    , ToIni(..)
     , auto
     , auto'
     , destination'
@@ -62,11 +61,10 @@ module Hkl.Binoculars.Config
     , getPreConfig
     , mergeIni
     , readConfig
-    , serializeConfig
     ) where
 
 
-import           Control.Applicative               (many, (<|>))
+import           Control.Applicative               (many, optional, (<|>))
 import           Control.Lens                      (makeLenses)
 import           Control.Monad.Catch               (MonadThrow, throwM)
 import           Control.Monad.Catch.Pure          (runCatch)
@@ -88,30 +86,30 @@ import           Data.Ini.Config.Bidir             (FieldValue (..), IniSpec,
                                                     text, (.=))
 import           Data.List                         (find, isInfixOf, length)
 import           Data.List.NonEmpty                (NonEmpty (..), map)
-import           Data.String                       (IsString)
+import           Data.Maybe                        (catMaybes)
 import           Data.Text                         (Text, breakOn, cons, drop,
                                                     empty, findIndex,
                                                     intercalate, length, lines,
                                                     pack, replace, singleton,
                                                     strip, take, takeWhile,
                                                     toLower, unlines, unpack,
-                                                    unwords)
+                                                    unwords, isInfixOf)
 import           Data.Text.IO                      (readFile)
 import           Data.Typeable                     (Proxy (..), Typeable,
                                                     typeRep)
 import           GHC.Conc                          (getNumCapabilities,
                                                     getNumProcessors)
 import           GHC.Exts                          (IsList (..))
+import           GHC.Generics                      (Generic)
 import           Numeric.Interval                  (Interval, empty, hull, inf,
                                                     singleton, singular, sup,
                                                     (...))
 import           Numeric.Units.Dimensional.NonSI   (angstrom)
 import           Numeric.Units.Dimensional.Prelude (Length, degree, meter, (*~),
                                                     (/~))
-import           Path                              (Abs, Dir, File, Path, Rel,
-                                                    fileExtension, filename,
-                                                    fromAbsDir, parseAbsDir,
-                                                    toFilePath)
+import           Path                              (Abs, Dir, File, Path,
+                                                    fileExtension, fromAbsDir,
+                                                    parseAbsDir, toFilePath)
 import           Path.IO                           (getCurrentDir, walkDirAccum)
 import           System.Directory                  (doesPathExist)
 import           System.FilePath                   (splitExtensions)
@@ -237,16 +235,11 @@ instance HasFieldValue (Int, Int) where
 
 -- Class HasIniConfig
 
-data family Config (a :: ProjectionType)
-data family DataPath (a :: ProjectionType)
-data family Args (a :: ProjectionType)
-
 readConfig :: Maybe FilePath -> IO ConfigContent
 readConfig mf = do
   cfg <- readFile =<< case mf of
                        Nothing  -> getDataFileName "data/test/config_manip1.cfg"
                        (Just f) -> pure f
-  -- return $ ConfigContent cfg
   return $ ConfigContent $ unlines $ [fixHeader l | l <- lines cfg]
     where
       fixHeader :: Text -> Text
@@ -254,20 +247,21 @@ readConfig mf = do
         Nothing  -> l
         (Just n) -> take n l
 
-class HasIniConfig (a :: ProjectionType) where
+class HasIniConfig (a :: k) where
+  data Config a
+  data Args a
+
+  defaultConfig :: Config a
 
   getConfig :: ConfigContent
             -> Args a
             -> Capabilities
             -> Either String (Config a)
 
--- Class ToIni
+  toIni :: Config a -> Ini
 
-class ToIni a where
-  toIni :: a -> Ini
-
-serializeConfig :: ToIni a => a -> Text
-serializeConfig = printIni . toIni
+  serializeConfig :: Config a -> Text
+  serializeConfig = printIni . toIni
 
 mergeHash :: (Eq k, Hashable k, Semigroup v) => HashMap k v -> HashMap k v -> HashMap k v
 mergeHash = unionWith f
@@ -316,13 +310,14 @@ getCapabilities = Capabilities
 -- ConfigRange
 
 newtype ConfigRange = ConfigRange (NonEmpty InputRange)
-  deriving (Eq, Show, IsList)
+  deriving (Eq, Show)
+  deriving newtype (IsList)
 
 instance Arbitrary ConfigRange where
   arbitrary = ConfigRange <$> ((:|) <$> arbitrary <*> arbitrary)
 
 instance FieldEmitter ConfigRange where
-  fieldEmitter (ConfigRange is) = unwords . toList $ Data.List.NonEmpty.map fieldEmitter is
+  fieldEmitter (ConfigRange is) = unwords . GHC.Exts.toList $ Data.List.NonEmpty.map fieldEmitter is
 
 instance FieldParsable ConfigRange where
   fieldParser = ConfigRange <$> ((:|)
@@ -375,6 +370,8 @@ instance FieldEmitter HklBinocularsQCustomSubProjectionEnum where
   fieldEmitter HklBinocularsQCustomSubProjectionEnum'QyQzTimestamp        = "qy_qz_timestamp"
   fieldEmitter HklBinocularsQCustomSubProjectionEnum'TthAzimuth           = "tth_azimuth"
   fieldEmitter HklBinocularsQCustomSubProjectionEnum'QTimescan0           = "q_timescan0"
+  fieldEmitter HklBinocularsQCustomSubProjectionEnum'QScannumber          = "q_scannumber"
+  fieldEmitter HklBinocularsQCustomSubProjectionEnum'TthScannumber        = "tth_scannumber"
 
 instance FieldParsable HklBinocularsQCustomSubProjectionEnum where
   fieldParser = go . strip . uncomment . toLower =<< takeText
@@ -513,6 +510,7 @@ instance HasFieldValue InputTypeDeprecated where
   fieldvalue = parsable
 
 data InputType = CristalK6C
+               | DiffabsCirpad
                | MarsFlyscan
                | MarsSbs
                | SixsFlyMedH
@@ -534,6 +532,7 @@ instance Arbitrary InputType where
 
 instance FieldEmitter InputType where
   fieldEmitter CristalK6C        = "cristal:k6c"
+  fieldEmitter DiffabsCirpad     = "diffabs:cirpad"
   fieldEmitter MarsFlyscan       = "mars:flyscan"
   fieldEmitter MarsSbs           = "mars:sbs"
   fieldEmitter SixsFlyMedH       = "sixs:flymedh"
@@ -581,17 +580,24 @@ instance Arbitrary Limits where
 
 -- MaskLocation
 
-newtype MaskLocation = MaskLocation { unMaskLocation :: Text }
-    deriving (Eq, Show, IsString)
+data MaskLocation = MaskLocation Text
+                  | MaskLocation'Tmpl Text
+    deriving (Eq, Generic, Show)
+    deriving anyclass (FromJSON, ToJSON)
 
-instance Arbitrary MaskLocation where
-  arbitrary = pure $ MaskLocation "mask location"
+instance FieldEmitter MaskLocation where
+  fieldEmitter (MaskLocation t) = t
+  fieldEmitter (MaskLocation'Tmpl t) = t
+
+instance FieldParsable MaskLocation where
+  fieldParser = do
+    t <- takeText
+    pure $ if "{scannumber:" `Data.Text.isInfixOf` t
+           then MaskLocation'Tmpl t
+           else MaskLocation t
 
 instance HasFieldValue MaskLocation where
-  fieldvalue = FieldValue
-    { fvParse = mapRight MaskLocation . fvParse text
-    , fvEmit = \(MaskLocation m) -> fvEmit text m
-    }
+  fieldvalue = parsable
 
 -- Meter
 
@@ -804,7 +810,7 @@ limitsP' = Limits
            <*> lim
   where
     lim :: Parser (Maybe Double)
-    lim = (Just <$> double) <|> return Nothing
+    lim = optional double
 
 showLimit :: Maybe Double -> Text
 showLimit (Just l) = pack $ printf "%f" l
@@ -815,7 +821,7 @@ showLimits (Limits f t) = showLimit f <> Data.Text.singleton ':' <> showLimit t
 
 instance (FieldEmitter (RLimits sh)) where
   fieldEmitter ls = Data.Text.singleton '['
-                    <> intercalate "," (Prelude.map showLimits (toList ls))
+                    <> intercalate "," (Prelude.map showLimits (GHC.Exts.toList ls))
                     <> Data.Text.singleton ']'
 
 instance (FieldParsable (RLimits DIM2)) where
@@ -939,52 +945,76 @@ destination' proj msub (ConfigRange rs) ml dtmpl overwrite =
         intervals = Data.List.NonEmpty.map unInputRange rs
 
 isHdf5 :: Path Abs File -> Bool
-isHdf5 p = case (fileExtension p :: Maybe [Char]) of
-             Nothing    -> False
-             (Just ext) -> ext `elem` [".h5", ".nxs"]
+isHdf5 p =
+  case (fileExtension p :: Maybe [Char]) of
+    Nothing  -> False
+    Just ext -> ext `elem` [".nxs"]
 
-isInConfigRange :: Maybe InputTmpl -> Maybe ConfigRange ->  Path Abs File -> Bool
-isInConfigRange mtmpl mr f
-  = case mr of
-      (Just (ConfigRange rs)) -> do
-        let tmpl = maybe "%05d" (unpack . unInputTmpl) mtmpl
-        any (isInInputRange (filename f) tmpl) rs
-      Nothing -> True
-  where
-    matchIndex :: Path Rel File -> String -> Int -> Bool
-    matchIndex p tmpl n = printf tmpl n `isInfixOf` toFilePath p
+-- isInConfigRange :: Maybe InputTmpl -> Maybe ConfigRange -> FilePath -> Bool
+-- isInConfigRange mtmpl mr f
+--   = case mr of
+--       (Just (ConfigRange rs)) -> do
+--         let tmpl = maybe "%05d" (unpack . unInputTmpl) mtmpl
+--         any (isInInputRange (filename f) tmpl) rs
+--       Nothing -> True
+--   where
+--     matchIndex :: Path Rel File -> String -> Int -> Bool
+--     matchIndex p tmpl n = printf tmpl n `isInfixOf` toFilePath p
 
-    isInInputRange :: Path Rel File -> String -> InputRange -> Bool
-    isInInputRange p tmpl (InputRange i) = any (matchIndex p tmpl) [inf i .. sup i]
+--     isInInputRange :: Path Rel File -> String -> InputRange -> Bool
+--     isInInputRange p tmpl (InputRange i) = any (matchIndex p tmpl) [inf i .. sup i]
+
+--                  return $ filter (\f -> all ($ f) filters) sfs)
+
+matchInputRange' :: String -> Path Abs File -> [Int] -> [Maybe ScanFilePath]
+matchInputRange' tmpl f is =
+  case find (\i -> printf tmpl i `Data.List.isInfixOf` (toFilePath f)) is of
+    Nothing -> []
+    Just i  -> [Just (ScanFilePath f (Scannumber i))]
+
+matchInputRange :: String -> Path Abs File ->  InputRange -> [Maybe ScanFilePath]
+matchInputRange tmpl f (InputRange ir)
+  = matchInputRange' tmpl f [inf ir .. sup ir]
+
+matchConfigRange' :: String -> ConfigRange -> Path Abs File -> [Maybe ScanFilePath]
+matchConfigRange' tmpl (ConfigRange is) f = concatMap (matchInputRange tmpl f) is
+
+matchConfigRange :: Maybe InputTmpl -> ConfigRange -> [Path Abs File] -> [Maybe ScanFilePath]
+matchConfigRange mt r fs =
+  let tmpl = maybe "%05d" (unpack . unInputTmpl) mt
+  in concatMap (matchConfigRange' tmpl r) fs
 
 files :: (MonadThrow m, MonadIO m)
        => Maybe (Path Abs Dir)
-       -> Maybe ConfigRange
+       -> ConfigRange
        -> Maybe InputTmpl
-       -> m [Path Abs File]
-files md mr mt
-  = do
-  let filters = [ isHdf5
-                , isInConfigRange mt mr
-                ]
+       -> m [ScanFilePath]
+files md mr mt =
+  do dir <- case md of
+             Nothing -> getCurrentDir
+             Just d  -> pure d
 
-  dir <- case md of
-        Nothing  -> getCurrentDir
-        (Just d) -> pure d
+     fs <- walkDirAccum Nothing
+          (\_root _dirs fs ->
+              do let fss = filter isHdf5 fs
+                 return $ matchConfigRange mt mr fss)
+          dir
 
-  fs <- walkDirAccum Nothing
-       (\_root _dirs fs -> return $ filter (\f -> all ($ f) filters) fs)
-       dir
+     if null fs
+       then throwM (NoDataFilesUnderTheGivenDirectory dir)
+       else return $ catMaybes fs
 
-  if null fs
-    then throwM (NoDataFilesUnderTheGivenDirectory dir)
-    else return fs
-
-getMask :: (MonadThrow m, MonadIO m) => Maybe MaskLocation -> Detector Hkl DIM2 -> m (Maybe Mask)
-getMask ml d = case ml of
-                Nothing          -> return Nothing
-                (Just "default") -> Just <$> getDetectorDefaultMask d
-                (Just fname)     -> Just <$> getDetectorMask d (unMaskLocation fname)
+getMask :: (MonadThrow m, MonadIO m) => Maybe MaskLocation -> Detector Hkl DIM2 -> Scannumber -> m (Maybe Mask)
+getMask ml d (Scannumber i)
+  = case ml of
+      Nothing          -> return Nothing
+      (Just (MaskLocation "default")) -> Just <$> getDetectorDefaultMask d
+      (Just (MaskLocation fname))     -> Just <$> getDetectorMask d fname
+      (Just (MaskLocation'Tmpl tmpl)) -> do
+        let tmpl1 = replace "{scannumber:" "%" tmpl
+        let tmpl2 = replace "}" "" tmpl1
+        let fname = pack $ printf (unpack tmpl2) i
+        Just <$> getDetectorMask d fname
 
 getPreConfig' :: ConfigContent -> Either String BinocularsPreConfig
 getPreConfig' (ConfigContent cfg) = do
