@@ -6,6 +6,7 @@
 {-# LANGUAGE MultiParamTypeClasses #-}
 {-# LANGUAGE OverloadedStrings     #-}
 {-# LANGUAGE RankNTypes            #-}
+{-# LANGUAGE RecordWildCards       #-}
 {-# LANGUAGE ScopedTypeVariables   #-}
 {-# LANGUAGE TypeFamilies          #-}
 
@@ -23,18 +24,16 @@
 -}
 
 module Hkl.Binoculars.Config.Sample
-    ( BinocularsConfig'Sample(..)
-    , default'BinocularsConfig'Sample
+    ( Args(..)
+    , Config(..)
     , default'DataSourcePath'Sample
     , guess'DataSourcePath'Sample
     , overload'DataSourcePath'Sample
-    , parse'BinocularsConfig'Sample
     ) where
 
 import           Data.Aeson                   (FromJSON, ToJSON)
 import           Data.HashMap.Lazy            (fromList)
 import           Data.Ini                     (Ini (..))
-import           Data.Text                    (Text)
 import           GHC.Generics                 (Generic)
 
 import           Hkl.Binoculars.Config
@@ -121,7 +120,7 @@ default'DataSourcePath'Sample = DataSourcePath'Sample
   (DataSourcePath'Degree(hdf5p $ grouppat 0 $ datasetp "SIXS/I14-C-CX2__EX__DIFF-UHV__#1/Uz"))
 
 
-overload'DataSourcePath'Sample :: BinocularsConfig'Sample
+overload'DataSourcePath'Sample :: Config Sample
                                -> DataSourcePath Sample
                                -> DataSourcePath Sample
 overload'DataSourcePath'Sample (BinocularsConfig'Sample ma mb mc malpha mbeta mgamma mux muy muz) (DataSourcePath'Sample pa pb pc palpha pbeta pgamma pux puy puz)
@@ -140,8 +139,8 @@ overload'DataSourcePath'Sample c (DataSourcePath'Sample'Or l r)
     (overload'DataSourcePath'Sample c l)
     (overload'DataSourcePath'Sample c r)
 
-guess'DataSourcePath'Sample :: BinocularsConfig'Common
-                           -> BinocularsConfig'Sample
+guess'DataSourcePath'Sample :: Config Common
+                           -> Config Sample
                            -> DataSourcePath Sample
 guess'DataSourcePath'Sample common sample =
   do let inputType = binocularsConfig'Common'InputType common
@@ -168,6 +167,7 @@ guess'DataSourcePath'Sample common sample =
            (DataSourcePath'Degree(hdf5p $ grouppat 0 $ groupp beamline $ groupp device $ datasetp "u_y"))
            (DataSourcePath'Degree(hdf5p $ grouppat 0 $ groupp beamline $ groupp device $ datasetp "u_z"))
      let cristalSamplePath = samplePath' "NOBEAMLINE" "NOBEAMLINE"
+     let diffabsSamplePath = samplePath' "DIFFABS" undefined
      let marsSamplePath = sampleMarsPath "MARS" "d03-1-cx2__ex__dif-cm_#1"
      let medHSamplePath = samplePath' "SIXS" "i14-c-cx1-ex-cm-med.h"
      let medVSamplePath = samplePath' "SIXS" "i14-c-cx1-ex-cm-med.v"
@@ -177,6 +177,7 @@ guess'DataSourcePath'Sample common sample =
 
      let samplePath = case inputType of
                         CristalK6C        -> cristalSamplePath
+                        DiffabsCirpad     -> diffabsSamplePath
                         MarsFlyscan       -> marsSamplePath
                         MarsSbs           -> marsSamplePath
                         SixsFlyMedH       -> medHSamplePath
@@ -198,35 +199,38 @@ guess'DataSourcePath'Sample common sample =
 -- Config --
 ------------
 
-data BinocularsConfig'Sample
-  = BinocularsConfig'Sample
-    { binocularsConfig'Sample'A     :: Maybe Double
-    , binocularsConfig'Sample'B     :: Maybe Double
-    , binocularsConfig'Sample'C     :: Maybe Double
-    , binocularsConfig'Sample'Alpha :: Maybe Degree
-    , binocularsConfig'Sample'Beta  :: Maybe Degree
-    , binocularsConfig'Sample'Gamma :: Maybe Degree
-    , binocularsConfig'Sample'Ux    :: Maybe Degree
-    , binocularsConfig'Sample'Uy    :: Maybe Degree
-    , binocularsConfig'Sample'Uz    :: Maybe Degree
-} deriving (Eq, Show, Generic)
+instance HasIniConfig Sample where
 
-default'BinocularsConfig'Sample :: BinocularsConfig'Sample
-default'BinocularsConfig'Sample
-  = BinocularsConfig'Sample
-    { binocularsConfig'Sample'A = Nothing
-    , binocularsConfig'Sample'B = Nothing
-    , binocularsConfig'Sample'C = Nothing
-    , binocularsConfig'Sample'Alpha = Nothing
-    , binocularsConfig'Sample'Beta = Nothing
-    , binocularsConfig'Sample'Gamma = Nothing
-    , binocularsConfig'Sample'Ux = Nothing
-    , binocularsConfig'Sample'Uy = Nothing
-    , binocularsConfig'Sample'Uz = Nothing
-    }
+    data Config Sample
+        = BinocularsConfig'Sample
+          { binocularsConfig'Sample'A     :: Maybe Double
+          , binocularsConfig'Sample'B     :: Maybe Double
+          , binocularsConfig'Sample'C     :: Maybe Double
+          , binocularsConfig'Sample'Alpha :: Maybe Degree
+          , binocularsConfig'Sample'Beta  :: Maybe Degree
+          , binocularsConfig'Sample'Gamma :: Maybe Degree
+          , binocularsConfig'Sample'Ux    :: Maybe Degree
+          , binocularsConfig'Sample'Uy    :: Maybe Degree
+          , binocularsConfig'Sample'Uz    :: Maybe Degree
+          } deriving (Eq, Show, Generic)
 
-instance ToIni BinocularsConfig'Sample where
-  toIni c = Ini { iniSections = fromList [ ("input", elemFMbDef "a" binocularsConfig'Sample'A c default'BinocularsConfig'Sample
+    data Args Sample
+        = Args'Sample
+
+    defaultConfig
+        = BinocularsConfig'Sample
+          { binocularsConfig'Sample'A = Nothing
+          , binocularsConfig'Sample'B = Nothing
+          , binocularsConfig'Sample'C = Nothing
+          , binocularsConfig'Sample'Alpha = Nothing
+          , binocularsConfig'Sample'Beta = Nothing
+          , binocularsConfig'Sample'Gamma = Nothing
+          , binocularsConfig'Sample'Ux = Nothing
+          , binocularsConfig'Sample'Uy = Nothing
+          , binocularsConfig'Sample'Uz = Nothing
+          }
+
+    toIni c = Ini { iniSections = fromList [ ("input", elemFMbDef "a" binocularsConfig'Sample'A c defaultConfig
                                                      [ "`a` parameter of the sample lattice (same unit than the wavelength)."
                                                      , ""
                                                      , "This parameter with the 5 others, `b`, `c`, `alpha`, `beta` and `gamma`"
@@ -235,7 +239,7 @@ instance ToIni BinocularsConfig'Sample where
                                                      , " `<not set>` - read `a` from the data file."
                                                      , " `a`         - override `a` with this value."
                                                      ]
-                                                     <> elemFMbDef "b" binocularsConfig'Sample'B c default'BinocularsConfig'Sample
+                                                     <> elemFMbDef "b" binocularsConfig'Sample'B c defaultConfig
                                                      [ "`b` parameter of the sample lattice (same unit than the wavelength)."
                                                      , ""
                                                      , "This parameter with the 5 others, `a`, `c`, `alpha`, `beta` and `gamma`"
@@ -244,7 +248,7 @@ instance ToIni BinocularsConfig'Sample where
                                                      , " `<not set>` - read `b` from the data file."
                                                      , " `b`         - override `b` with this value."
                                                      ]
-                                                     <> elemFMbDef "c" binocularsConfig'Sample'C c default'BinocularsConfig'Sample
+                                                     <> elemFMbDef "c" binocularsConfig'Sample'C c defaultConfig
                                                      [ "`c` parameter of the sample lattice (same unit than the wavelength)."
                                                      , ""
                                                      , "This parameter with the 5 others, `a`, `b`, `alpha`, `beta` and `gamma`"
@@ -253,7 +257,7 @@ instance ToIni BinocularsConfig'Sample where
                                                      , " `<not set>` - read `c` from the data file."
                                                      , " `c`         - override `c` with this value."
                                                      ]
-                                                     <> elemFMbDef "alpha" binocularsConfig'Sample'Alpha c default'BinocularsConfig'Sample
+                                                     <> elemFMbDef "alpha" binocularsConfig'Sample'Alpha c defaultConfig
                                                      [ "`alpha` parameter of the sample lattice (Degree)."
                                                      , ""
                                                      , "This parameter with the 5 others, `a`, `b`, `c`, `beta` and `gamma`"
@@ -262,7 +266,7 @@ instance ToIni BinocularsConfig'Sample where
                                                      , " `<not set>` - read `alpha` from the data file."
                                                      , " `alpha`     - override `alpha` with this value."
                                                      ]
-                                                     <> elemFMbDef "beta" binocularsConfig'Sample'Beta c default'BinocularsConfig'Sample
+                                                     <> elemFMbDef "beta" binocularsConfig'Sample'Beta c defaultConfig
                                                      [ "`beta` parameter of the sample lattice (Degree)."
                                                      , ""
                                                      , "This parameter with the 5 others, `a`, `b`, `c`, `alpha`, and `gamma`"
@@ -271,7 +275,7 @@ instance ToIni BinocularsConfig'Sample where
                                                      , " `<not set>` - read `beta` from the data file."
                                                      , " `beta`      - override `beta` with this value."
                                                      ]
-                                                     <> elemFMbDef "gamma" binocularsConfig'Sample'Gamma c default'BinocularsConfig'Sample
+                                                     <> elemFMbDef "gamma" binocularsConfig'Sample'Gamma c defaultConfig
                                                      [ "`gamma` parameter of the sample lattice (Degree)."
                                                      , ""
                                                      , "This parameter with the 5 others, `a`, `b`, `c`, `alpha` and `beta`"
@@ -280,7 +284,7 @@ instance ToIni BinocularsConfig'Sample where
                                                      , " `<not set>` - read `gamma` from the data file."
                                                      , " `gamma`     - override `gamma` with this value."
                                                      ]
-                                                     <> elemFMbDef "ux" binocularsConfig'Sample'Ux c default'BinocularsConfig'Sample
+                                                     <> elemFMbDef "ux" binocularsConfig'Sample'Ux c defaultConfig
                                                      [ "`ux` rotation of the sample around the x axis"
                                                      , ""
                                                      , "`ux`, `uy`, `uz` are the eulerian angles, which define"
@@ -294,7 +298,7 @@ instance ToIni BinocularsConfig'Sample where
                                                      , " `<not set>` - read `ux` from the data file."
                                                      , " `ux`        - override `ux` with this value."
                                                      ]
-                                                     <> elemFMbDef "uy" binocularsConfig'Sample'Uy c default'BinocularsConfig'Sample
+                                                     <> elemFMbDef "uy" binocularsConfig'Sample'Uy c defaultConfig
                                                      [ "`uy` rotation of the sample around the y axis"
                                                      , ""
                                                      , "`ux`, `uy`, `uz` are the eulerian angles, which define"
@@ -308,7 +312,7 @@ instance ToIni BinocularsConfig'Sample where
                                                      , " `<not set>` - read `uy` from the data file."
                                                      , " `uy`        - override `uy` with this value."
                                                      ]
-                                                     <> elemFMbDef "uz" binocularsConfig'Sample'Uz c default'BinocularsConfig'Sample
+                                                     <> elemFMbDef "uz" binocularsConfig'Sample'Uz c defaultConfig
                                                      [ "`uz` rotation of the sample around the z axis"
                                                      , ""
                                                      , "`ux`, `uy`, `uz` are the eulerian angles, which define"
@@ -327,15 +331,14 @@ instance ToIni BinocularsConfig'Sample where
                 , iniGlobals = []
                 }
 
-parse'BinocularsConfig'Sample :: Text -> Either String BinocularsConfig'Sample
-parse'BinocularsConfig'Sample cfg
-  = BinocularsConfig'Sample
-    <$> parseMb cfg "input" "a"
-    <*> parseMb cfg "input" "b"
-    <*> parseMb cfg "input" "c"
-    <*> parseMb cfg "input" "alpha"
-    <*> parseMb cfg "input" "beta"
-    <*> parseMb cfg "input" "gamma"
-    <*> parseMb cfg "input" "ux"
-    <*> parseMb cfg "input" "uy"
-    <*> parseMb cfg "input" "uz"
+    getConfig (ConfigContent cfg) _ _
+        = do binocularsConfig'Sample'A <- parseMb cfg "input" "a"
+             binocularsConfig'Sample'B <- parseMb cfg "input" "b"
+             binocularsConfig'Sample'C <- parseMb cfg "input" "c"
+             binocularsConfig'Sample'Alpha <- parseMb cfg "input" "alpha"
+             binocularsConfig'Sample'Beta <- parseMb cfg "input" "beta"
+             binocularsConfig'Sample'Gamma <- parseMb cfg "input" "gamma"
+             binocularsConfig'Sample'Ux <- parseMb cfg "input" "ux"
+             binocularsConfig'Sample'Uy <- parseMb cfg "input" "uy"
+             binocularsConfig'Sample'Uz <- parseMb cfg "input" "uz"
+             pure BinocularsConfig'Sample{..}
