@@ -2,6 +2,7 @@
 {-# LANGUAGE FlexibleContexts    #-}
 {-# LANGUAGE FlexibleInstances   #-}
 {-# LANGUAGE OverloadedStrings   #-}
+{-# LANGUAGE RecordWildCards     #-}
 {-# LANGUAGE ScopedTypeVariables #-}
 {-# LANGUAGE TypeFamilies        #-}
 
@@ -60,69 +61,64 @@ import           Hkl.Utils
 -- Config --
 ------------
 
-data instance Config 'AnglesProjection = BinocularsConfig'Angles
-  { binocularsConfig'Angles'Common                 :: BinocularsConfig'Common
-  , binocularsConfig'Angles'ProjectionType         :: ProjectionType
-  , binocularsConfig'Angles'ProjectionResolution   :: Resolutions DIM3
-  , binocularsConfig'Angles'ProjectionLimits       :: Maybe (RLimits DIM3)
-  , binocularsConfig'Angles'SampleAxis             :: SampleAxis
-  , binocularsConfig'Angles'DataPath               :: DataSourcePath DataFrameQCustom
- } deriving (Show)
-
-
-default'BinocularsConfig'Angles :: Config 'AnglesProjection
-default'BinocularsConfig'Angles
-  = BinocularsConfig'Angles
-    { binocularsConfig'Angles'Common = default'BinocularsConfig'Common
-    , binocularsConfig'Angles'ProjectionType = AnglesProjection
-    , binocularsConfig'Angles'ProjectionResolution = Resolutions3 1 1 1
-    , binocularsConfig'Angles'ProjectionLimits = Nothing
-    , binocularsConfig'Angles'DataPath = default'DataSourcePath'DataFrameQCustom
-    , binocularsConfig'Angles'SampleAxis = SampleAxis "omega"
-    }
-
-newtype instance Args 'AnglesProjection = Args'AnglesProjection (Maybe ConfigRange)
-
 instance HasIniConfig 'AnglesProjection where
+  data instance Config 'AnglesProjection
+      = BinocularsConfig'Angles
+        { binocularsConfig'Angles'Common                 :: Config Common
+        , binocularsConfig'Angles'ProjectionType         :: ProjectionType
+        , binocularsConfig'Angles'ProjectionResolution   :: Resolutions DIM3
+        , binocularsConfig'Angles'ProjectionLimits       :: Maybe (RLimits DIM3)
+        , binocularsConfig'Angles'SampleAxis             :: SampleAxis
+        , binocularsConfig'Angles'DataPath               :: DataSourcePath DataFrameQCustom
+        } deriving (Show)
 
-  getConfig content@(ConfigContent cfg) (Args'AnglesProjection mr) capabilities = do
-    common <- parse'BinocularsConfig'Common cfg mr capabilities
-    projectionType <- parseFDef cfg "projection" "type" (binocularsConfig'Angles'ProjectionType default'BinocularsConfig'Angles)
-    BinocularsConfig'Angles
-      <$> pure common
-      <*> pure projectionType
-      <*> parseFDef cfg "projection" "resolution" (binocularsConfig'Angles'ProjectionResolution default'BinocularsConfig'Angles)
-      <*> parseMb cfg "projection" "limits"
-      <*> (parseFDef cfg "input" "sample_axis" $ case projectionType of
-                                                   AnglesProjection   -> SampleAxis "omega"
-                                                   Angles2Projection  -> SampleAxis "mu"
-                                                   HklProjection      -> undefined
-                                                   QCustomProjection  -> undefined
-                                                   QIndexProjection   -> undefined
-                                                   QparQperProjection -> undefined
-                                                   QxQyQzProjection   -> undefined
-                                                   RealSpaceProjection -> undefined
-                                                   PixelsProjection -> undefined
-                                                   TestProjection  -> undefined)
-      <*> (pure $ eitherF (const $ guess'DataSourcePath'DataFrameQCustom common Nothing content) (parse' cfg "input" "datapath")
-           (\md -> case md of
-                    Nothing -> guess'DataSourcePath'DataFrameQCustom common Nothing content
-                    Just d  ->  overload'DataSourcePath'DataFrameQCustom common Nothing d))
+  newtype Args 'AnglesProjection
+      = Args'AnglesProjection (Maybe ConfigRange)
 
+  defaultConfig
+      = BinocularsConfig'Angles
+        { binocularsConfig'Angles'Common = defaultConfig
+        , binocularsConfig'Angles'ProjectionType = AnglesProjection
+        , binocularsConfig'Angles'ProjectionResolution = Resolutions3 1 1 1
+        , binocularsConfig'Angles'ProjectionLimits = Nothing
+        , binocularsConfig'Angles'DataPath = default'DataSourcePath'DataFrameQCustom
+        , binocularsConfig'Angles'SampleAxis = SampleAxis "omega"
+        }
+
+  getConfig content@(ConfigContent cfg) (Args'AnglesProjection mr) capabilities
+      = do binocularsConfig'Angles'Common <- getConfig content (Args'Common mr) capabilities
+           binocularsConfig'Angles'ProjectionType <- parseFDef cfg "projection" "type" (binocularsConfig'Angles'ProjectionType defaultConfig)
+           binocularsConfig'Angles'ProjectionResolution <- parseFDef cfg "projection" "resolution" (binocularsConfig'Angles'ProjectionResolution defaultConfig)
+           binocularsConfig'Angles'ProjectionLimits <- parseMb cfg "projection" "limits"
+           binocularsConfig'Angles'SampleAxis <- parseFDef cfg "input" "sample_axis" $ case binocularsConfig'Angles'ProjectionType of
+                                                                                        AnglesProjection   -> SampleAxis "omega"
+                                                                                        Angles2Projection  -> SampleAxis "mu"
+                                                                                        HklProjection      -> undefined
+                                                                                        QCustomProjection  -> undefined
+                                                                                        QIndexProjection   -> undefined
+                                                                                        QparQperProjection -> undefined
+                                                                                        QxQyQzProjection   -> undefined
+                                                                                        RealSpaceProjection -> undefined
+                                                                                        PixelsProjection -> undefined
+                                                                                        TestProjection  -> undefined
+           binocularsConfig'Angles'DataPath <- pure $ eitherF (const $ guess'DataSourcePath'DataFrameQCustom binocularsConfig'Angles'Common Nothing content) (parse' cfg "input" "datapath")
+                                              (\md -> case md of
+                                                       Nothing -> guess'DataSourcePath'DataFrameQCustom binocularsConfig'Angles'Common Nothing content
+                                                       Just d  ->  overload'DataSourcePath'DataFrameQCustom binocularsConfig'Angles'Common Nothing d)
+           pure BinocularsConfig'Angles{..}
 
-instance ToIni (Config 'AnglesProjection) where
   toIni c = toIni (binocularsConfig'Angles'Common c)
             `mergeIni`
-            Ini { iniSections = fromList [ ("input", elemFDef' "datapath" binocularsConfig'Angles'DataPath c default'BinocularsConfig'Angles
-                                                     <> elemFDef "sample_axis" binocularsConfig'Angles'SampleAxis c default'BinocularsConfig'Angles
+            Ini { iniSections = fromList [ ("input", elemFDef' "datapath" binocularsConfig'Angles'DataPath c defaultConfig
+                                                     <> elemFDef "sample_axis" binocularsConfig'Angles'SampleAxis c defaultConfig
                                                      [ "the name of the sample axis"
                                                      , ""
                                                      , "default value: `omega`"
                                                      ]
                                             )
-                                         , ("projection", elemFDef' "type" binocularsConfig'Angles'ProjectionType c default'BinocularsConfig'Angles
-                                                          <> elemFDef' "resolution" binocularsConfig'Angles'ProjectionResolution c default'BinocularsConfig'Angles
-                                                          <> elemFMbDef' "limits" binocularsConfig'Angles'ProjectionLimits c default'BinocularsConfig'Angles
+                                         , ("projection", elemFDef' "type" binocularsConfig'Angles'ProjectionType c defaultConfig
+                                                          <> elemFDef' "resolution" binocularsConfig'Angles'ProjectionResolution c defaultConfig
+                                                          <> elemFMbDef' "limits" binocularsConfig'Angles'ProjectionLimits c defaultConfig
                                            )]
                 , iniGlobals = []
                 }
@@ -132,14 +128,14 @@ instance ToIni (Config 'AnglesProjection) where
 -------------------------
 
 {-# INLINE spaceAngles #-}
-spaceAngles :: Detector a DIM2 -> Array F DIM3 Double -> Resolutions DIM3 -> Maybe Mask -> Maybe (RLimits DIM3) -> SampleAxis -> Space DIM2 -> DataFrameQCustom -> IO (DataFrameSpace DIM2)
-spaceAngles det pixels rs mmask' mlimits sAxis space@(Space fSpace) (DataFrameQCustom att g img _ _) =
+spaceAngles :: Detector a DIM2 -> Array F DIM3 Double -> Resolutions DIM3 -> Maybe (RLimits DIM3) -> SampleAxis -> Space DIM2 -> DataFrameQCustom -> IO (DataFrameSpace DIM2)
+spaceAngles det pixels rs mlimits sAxis space@(Space fSpace) (DataFrameQCustom att g img mmask _ _ _) =
   withNPixels det $ \nPixels ->
   withGeometry g $ \geometry ->
   withForeignPtr (toForeignPtr pixels) $ \pix ->
   withResolutions rs $ \nr r ->
   withPixelsDims pixels $ \ndim dims ->
-  withMaybeMask mmask' $ \ mask'' ->
+  withMaybeMask mmask $ \ mask'' ->
   withMaybeLimits mlimits rs $ \nlimits limits ->
   withSampleAxis sAxis $ \sampleAxis ->
   withForeignPtr fSpace $ \pSpace -> do
@@ -176,7 +172,6 @@ processAnglesP = do
   let inputRange = binocularsConfig'Common'InputRange common
   let nexusDir = binocularsConfig'Common'Nexusdir common
   let tmpl = binocularsConfig'Common'Tmpl common
-  let maskMatrix = binocularsConfig'Common'Maskmatrix common
   let mSkipFirstPoints = binocularsConfig'Common'SkipFirstPoints common
   let mSkipLastPoints = binocularsConfig'Common'SkipLastPoints common
 
@@ -189,8 +184,7 @@ processAnglesP = do
 
   -- built from the config
   output' <- liftIO $ destination' projectionType Nothing inputRange mlimits destination overwrite
-  filenames <- InputFn'List <$> files nexusDir (Just inputRange) tmpl
-  mask' <- getMask maskMatrix det
+  filenames <- InputFn'List <$> files nexusDir inputRange tmpl
   pixels <- liftIO $ getPixelsCoordinates det centralPixel' sampleDetectorDistance detrot Normalisation
   let fns = concatMap (replicate 1) (toList filenames)
   chunks <- liftIO $ runSafeT $ toListM $ each fns >-> chunkP mSkipFirstPoints mSkipLastPoints datapaths
@@ -211,7 +205,7 @@ processAnglesP = do
     each chunks
     >-> Pipes.Prelude.map (\(Chunk fn f t) -> (fn, [f, quot (f + t) 4, quot (f + t) 4 * 2, quot (f + t) 4 * 3, t]))
     >-> framesP datapaths
-    >-> project det 3 (spaceAngles det pixels res mask' mlimits sampleAxis)
+    >-> project det 3 (spaceAngles det pixels res mlimits sampleAxis)
     >-> accumulateP c
 
   logDebugN "stop gessing final cube size"
@@ -226,8 +220,8 @@ processAnglesP = do
                              each job
                              >-> Pipes.Prelude.map (\(Chunk fn f t) -> (fn, [f..t]))
                              >-> framesP datapaths
-                             >-> Pipes.Prelude.filter (\(DataFrameQCustom _ _ img _ _) -> filterSumImage mImageSumMax img)
-                             >-> project det 3 (spaceAngles det pixels res mask' mlimits sampleAxis)
+                             >-> Pipes.Prelude.filter (\(DataFrameQCustom _ _ img _ _ _ _) -> filterSumImage mImageSumMax img)
+                             >-> project det 3 (spaceAngles det pixels res mlimits sampleAxis)
                              >-> tee (accumulateP c)
                              >-> progress pb
                          ) jobs
@@ -243,8 +237,8 @@ processAngles mf mr = cmd processAnglesP mf (Args'AnglesProjection mr)
 newAngles :: (MonadIO m, MonadLogger m, MonadThrow m)
           => Path Abs Dir -> m ()
 newAngles cwd = do
-  let conf = default'BinocularsConfig'Angles
-             { binocularsConfig'Angles'Common = default'BinocularsConfig'Common
+  let conf = defaultConfig
+             { binocularsConfig'Angles'Common = defaultConfig
                                                 { binocularsConfig'Common'Nexusdir = Just cwd }
              }
   liftIO $ Data.Text.IO.putStr $ serializeConfig conf
