@@ -5,6 +5,7 @@
 {-# LANGUAGE FlexibleInstances     #-}
 {-# LANGUAGE MultiParamTypeClasses #-}
 {-# LANGUAGE OverloadedStrings     #-}
+{-# LANGUAGE RecordWildCards       #-}
 {-# LANGUAGE ScopedTypeVariables   #-}
 {-# LANGUAGE StandaloneDeriving    #-}
 {-# LANGUAGE TypeFamilies          #-}
@@ -29,12 +30,12 @@ module Hkl.Binoculars.Projections.Test
     ) where
 
 import           Control.Concurrent.Async           (mapConcurrently)
+import           Control.Monad                      (forM_, forever)
 import           Control.Monad.Catch                (MonadThrow)
 import           Control.Monad.IO.Class             (MonadIO (liftIO))
 import           Control.Monad.Logger               (MonadLogger, logDebugN,
                                                      logInfoN)
-import           Control.Monad.Reader               (MonadReader, ask, forM_,
-                                                     forever)
+import           Control.Monad.Reader               (MonadReader, ask)
 import           Data.Aeson                         (FromJSON, ToJSON,
                                                      eitherDecode', encode)
 import           Data.ByteString.Lazy               (fromStrict, toStrict)
@@ -67,11 +68,9 @@ import           Hkl.C.Binoculars
 import           Hkl.DataSource
 import           Hkl.Detector
 import           Hkl.Geometry
-import           Hkl.H5
 import           Hkl.HKD
 import           Hkl.Image
 import           Hkl.Orphan                         ()
-import           Hkl.Pipes
 import           Hkl.Repa
 import           Hkl.Sample
 import           Hkl.Utils
@@ -114,33 +113,10 @@ instance HasFieldValue (DataFrameTest' DataSourcePath) where
 -- Config --
 ------------
 
-data instance Config 'TestProjection
-  = BinocularsConfig'Test
-    { binocularsConfig'Test'Common                 :: BinocularsConfig'Common
-    , binocularsConfig'Test'Sample                 :: BinocularsConfig'Sample
-    , binocularsConfig'Test'ProjectionType         :: ProjectionType
-    , binocularsConfig'Test'ProjectionResolution   :: Resolutions DIM3
-    , binocularsConfig'Test'ProjectionLimits       :: Maybe (RLimits DIM3)
-    , binocularsConfig'Test'DataPath               :: DataFrameTest' DataSourcePath
-    } deriving (Generic)
-
-newtype instance Args 'TestProjection = Args'TestProjection (Maybe ConfigRange)
-
-default'BinocularsConfig'Test :: Config 'TestProjection
-default'BinocularsConfig'Test
-  = BinocularsConfig'Test
-    { binocularsConfig'Test'Common = default'BinocularsConfig'Common
-    , binocularsConfig'Test'Sample = default'BinocularsConfig'Sample
-    , binocularsConfig'Test'ProjectionType = TestProjection
-    , binocularsConfig'Test'ProjectionResolution = Resolutions3 0.01 0.01 0.01
-    , binocularsConfig'Test'ProjectionLimits  = Nothing
-    , binocularsConfig'Test'DataPath = defaultDataSourcePath'DataFrameTest
-    }
-
-overload'DataSourcePath'DataFrameTest :: BinocularsConfig'Common
-                                     -> BinocularsConfig'Sample
-                                     -> DataFrameTest' DataSourcePath
-                                     -> DataFrameTest' DataSourcePath
+overload'DataSourcePath'DataFrameTest :: Config Common
+                                      -> Config Sample
+                                      -> DataFrameTest' DataSourcePath
+                                      -> DataFrameTest' DataSourcePath
 overload'DataSourcePath'DataFrameTest common sample (DataFrameTest qCustomPath samplePath)
   = DataFrameTest newQCustomPath newSamplePath
   where
@@ -148,31 +124,48 @@ overload'DataSourcePath'DataFrameTest common sample (DataFrameTest qCustomPath s
     newSamplePath = overload'DataSourcePath'Sample sample samplePath
 
 instance HasIniConfig 'TestProjection where
+  data Config 'TestProjection
+    = BinocularsConfig'Test
+      { binocularsConfig'Test'Common                 :: Config Common
+      , binocularsConfig'Test'Sample                 :: Config Sample
+      , binocularsConfig'Test'ProjectionType         :: ProjectionType
+      , binocularsConfig'Test'ProjectionResolution   :: Resolutions DIM3
+      , binocularsConfig'Test'ProjectionLimits       :: Maybe (RLimits DIM3)
+      , binocularsConfig'Test'DataPath               :: DataFrameTest' DataSourcePath
+      } deriving (Generic)
 
-  getConfig content@(ConfigContent cfg) (Args'TestProjection mr) capabilities = do
-    common <- parse'BinocularsConfig'Common cfg mr capabilities
-    sample <- parse'BinocularsConfig'Sample cfg
-    BinocularsConfig'Test
-      <$> pure common
-      <*> pure sample
-      <*> parseFDef cfg "projection" "type" (binocularsConfig'Test'ProjectionType default'BinocularsConfig'Test)
-      <*> parseFDef cfg "projection" "resolution" (binocularsConfig'Test'ProjectionResolution default'BinocularsConfig'Test)
-      <*> parseMb cfg "projection" "limits"
-      <*> (pure $ eitherF (const $ guess'DataSourcePath'DataFrameTest common sample content) (parse' cfg "input" "datapath")
-           (\md -> case md of
-                    Nothing -> guess'DataSourcePath'DataFrameTest common sample content
-                    Just d  ->  overload'DataSourcePath'DataFrameTest common sample d))
+  newtype Args 'TestProjection = Args'TestProjection (Maybe ConfigRange)
 
+  defaultConfig
+      = BinocularsConfig'Test
+        { binocularsConfig'Test'Common = defaultConfig
+        , binocularsConfig'Test'Sample = defaultConfig
+        , binocularsConfig'Test'ProjectionType = TestProjection
+        , binocularsConfig'Test'ProjectionResolution = Resolutions3 0.01 0.01 0.01
+        , binocularsConfig'Test'ProjectionLimits  = Nothing
+        , binocularsConfig'Test'DataPath = defaultDataSourcePath'DataFrameTest
+        }
+
+  getConfig content@(ConfigContent cfg) (Args'TestProjection mr) capabilities
+      = do binocularsConfig'Test'Common <- getConfig content (Args'Common mr) capabilities
+           binocularsConfig'Test'Sample <- getConfig content Args'Sample capabilities
+           binocularsConfig'Test'ProjectionType <- parseFDef cfg "projection" "type" (binocularsConfig'Test'ProjectionType defaultConfig)
+           binocularsConfig'Test'ProjectionResolution <- parseFDef cfg "projection" "resolution" (binocularsConfig'Test'ProjectionResolution defaultConfig)
+           binocularsConfig'Test'ProjectionLimits <- parseMb cfg "projection" "limits"
+           binocularsConfig'Test'DataPath <- (pure $ eitherF (const $ guess'DataSourcePath'DataFrameTest binocularsConfig'Test'Common binocularsConfig'Test'Sample content) (parse' cfg "input" "datapath")
+                                             (\md -> case md of
+                                                      Nothing -> guess'DataSourcePath'DataFrameTest binocularsConfig'Test'Common binocularsConfig'Test'Sample content
+                                                      Just d  ->  overload'DataSourcePath'DataFrameTest binocularsConfig'Test'Common binocularsConfig'Test'Sample d))
+           pure BinocularsConfig'Test{..}
 
-instance ToIni (Config 'TestProjection) where
   toIni c = toIni (binocularsConfig'Test'Common c)
             `mergeIni`
             toIni (binocularsConfig'Test'Sample c)
             `mergeIni`
-            Ini { iniSections = fromList [ ("input", elemFDef' "datapath" binocularsConfig'Test'DataPath c default'BinocularsConfig'Test)
-                                         , ("projection", elemFDef' "type" binocularsConfig'Test'ProjectionType c default'BinocularsConfig'Test
-                                                          <> elemFDef' "resolution" binocularsConfig'Test'ProjectionResolution c default'BinocularsConfig'Test
-                                                          <> elemFMbDef' "limits" binocularsConfig'Test'ProjectionLimits c default'BinocularsConfig'Test
+            Ini { iniSections = fromList [ ("input", elemFDef' "datapath" binocularsConfig'Test'DataPath c defaultConfig)
+                                         , ("projection", elemFDef' "type" binocularsConfig'Test'ProjectionType c defaultConfig
+                                                          <> elemFDef' "resolution" binocularsConfig'Test'ProjectionResolution c defaultConfig
+                                                          <> elemFMbDef' "limits" binocularsConfig'Test'ProjectionLimits c defaultConfig
                                            )
                                          ]
                 , iniGlobals = []
@@ -184,14 +177,14 @@ instance ToIni (Config 'TestProjection) where
 ----------------
 
 {-# INLINE spaceTest #-}
-spaceTest :: Detector b DIM2 -> Array F DIM3 Double -> Resolutions DIM3 -> Maybe Mask -> Maybe (RLimits DIM3) -> Bool -> Space DIM3 -> DataFrameTest' Identity -> IO (DataFrameSpace DIM3)
-spaceTest det pixels rs mmask' mlimits doPolarizationCorrection space@(Space fSpace) (DataFrameTest (DataFrameQCustom att g img _ _) samplePath) = do
+spaceTest :: Detector b DIM2 -> Array F DIM3 Double -> Resolutions DIM3 -> Maybe (RLimits DIM3) -> Bool -> Space DIM3 -> DataFrameTest' Identity -> IO (DataFrameSpace DIM3)
+spaceTest det pixels rs mlimits doPolarizationCorrection space@(Space fSpace) (DataFrameTest (DataFrameQCustom att g img mmask _ _ _) samplePath) = do
   withNPixels det $ \nPixels ->
     withGeometry g $ \geometry ->
     withSample samplePath $ \sample ->
     withForeignPtr (toForeignPtr pixels) $ \pix ->
     withResolutions rs $ \nr r ->
-    withMaybeMask mmask' $ \ mask'' ->
+    withMaybeMask mmask $ \ mask'' ->
     withPixelsDims pixels $ \ndim dims ->
     withMaybeLimits mlimits rs $ \nlimits limits ->
     withForeignPtr fSpace $ \pSpace -> do
@@ -227,7 +220,6 @@ processTestP = do
   let inputRange = binocularsConfig'Common'InputRange common
   let nexusDir = binocularsConfig'Common'Nexusdir common
   let tmpl = binocularsConfig'Common'Tmpl common
-  let maskMatrix = binocularsConfig'Common'Maskmatrix common
   let mSkipFirstPoints = binocularsConfig'Common'SkipFirstPoints common
   let mSkipLastPoints = binocularsConfig'Common'SkipLastPoints common
   let doPolarizationCorrection = binocularsConfig'Common'PolarizationCorrection common
@@ -241,8 +233,7 @@ processTestP = do
 
   -- built from the config
   output' <- liftIO $ destination' projectionType Nothing inputRange mlimits destination overwrite
-  filenames <- InputFn'List <$> files nexusDir (Just inputRange) tmpl
-  mask' <- getMask maskMatrix det
+  filenames <- InputFn'List <$> files nexusDir inputRange tmpl
   pixels <- liftIO $ getPixelsCoordinates det centralPixel' sampleDetectorDistance detrot NoNormalisation
 
   let fns = concatMap (replicate 1) (toList filenames)
@@ -264,7 +255,7 @@ processTestP = do
     each chunks
     >-> Pipes.Prelude.map (\(Chunk fn f t) -> (fn, [f, quot (f + t) 4, quot (f + t) 4 * 2, quot (f + t) 4 * 3, t]))
     >-> framesP datapaths
-    >-> project det 3 (spaceTest det pixels res mask' mlimits doPolarizationCorrection)
+    >-> project det 3 (spaceTest det pixels res mlimits doPolarizationCorrection)
     >-> accumulateP c
 
   logDebugN "stop gessing final cube size"
@@ -278,8 +269,8 @@ processTestP = do
                              >-> Pipes.Prelude.map (\(Chunk fn f t) -> (fn, [f..t]))
                              -- >-> tee Pipes.Prelude.print
                              >-> framesP datapaths
-                             >-> Pipes.Prelude.filter (\(DataFrameTest (DataFrameQCustom _ _ img _ _) _) -> filterSumImage mImageSumMax img)
-                             >-> project det 3 (spaceTest det pixels res mask' mlimits doPolarizationCorrection)
+                             >-> Pipes.Prelude.filter (\(DataFrameTest (DataFrameQCustom _ _ img _ _ _ _) _) -> filterSumImage mImageSumMax img)
+                             >-> project det 3 (spaceTest det pixels res mlimits doPolarizationCorrection)
                              >-> tee (accumulateP c)
                              >-> progress pb
                          ) jobs
@@ -293,7 +284,7 @@ instance ChunkP (DataFrameTest' DataSourcePath) where
 instance FramesP (DataFrameTest' DataSourcePath) (DataFrameTest' Identity) where
   framesP (DataFrameTest qcustom sample) = skipMalformed $ forever $ do
     (fp, js) <- await
-    withFileP (openFile' fp) $ \f ->
+    withScanFileP fp $ \f ->
       withDataSourceP f qcustom $ \qcustomAcq ->
       withDataSourceP f sample $ \sampleAcq ->
       forM_ js (\j -> tryYield ( DataFrameTest
@@ -305,10 +296,10 @@ instance FramesP (DataFrameTest' DataSourcePath) (DataFrameTest' Identity) where
 -- Inputs --
 ------------
 
-guess'DataSourcePath'DataFrameTest :: BinocularsConfig'Common
-                                  -> BinocularsConfig'Sample
-                                  -> ConfigContent
-                                  -> DataFrameTest' DataSourcePath
+guess'DataSourcePath'DataFrameTest :: Config Common
+                                   -> Config Sample
+                                   -> ConfigContent
+                                   -> DataFrameTest' DataSourcePath
 guess'DataSourcePath'DataFrameTest common sample content
   = DataFrameTest
     (guess'DataSourcePath'DataFrameQCustom common Nothing content)
@@ -324,8 +315,8 @@ processTest mf mr = cmd processTestP mf (Args'TestProjection mr)
 newTest :: (MonadIO m, MonadLogger m, MonadThrow m)
        => Path Abs Dir -> m ()
 newTest cwd = do
-  let conf = default'BinocularsConfig'Test
-             { binocularsConfig'Test'Common = default'BinocularsConfig'Common
+  let conf = defaultConfig
+             { binocularsConfig'Test'Common = defaultConfig
                                              { binocularsConfig'Common'Nexusdir = Just cwd }
              }
   liftIO $ Data.Text.IO.putStr $ serializeConfig conf
